{{
@file pjvm_demo.spin:

  This is a PASM version of Peter's Spin JVM.
  This code is compatible with much of the Javelin Stamp
  library at http://tech.groups.yahoo.com/group/JavelinCode
  (testing is still in progress, so there may be errors.)

  '------------------------------------------------
  The JVM provides 18KB for user Java bytecode.
  This is different from the Javelin which has 32KB.
  '------------------------------------------------

  '------------------------------------------------
  In the future, being able to define custom JNI devices
  is very desirable. LCD, touch screen, micro-ball mouse,
  keyboard/keypad, tone generator, etc... are possible.
  '------------------------------------------------

  '------------------------------------------------
  Standard configuration:
  6 COGs used; 2 free for user apps.

  1 for Spin.
  1 for JVM Runner.
  1 for JVM Core Interpreter.
  3 for VPs by default up to 4 or none at all.
  '------------------------------------------------

  '------------------------------------------------
  Minimum COG configuration:
  2 COGs used; 6 COGs free for user apps.
  Between 4 and 10 times slower than standard.

  1 for Spin.
  1 for JVM Core Interpreter.
  '------------------------------------------------

  '------------------------------------------------
  Notes:
  '------------------------------------------------
  Use FDX define to enable high speed serial - uses 1 COG
  Up to 8 VPs can be used at once - uses 4 COGs
  Garbage collection is not functional and disabled.
  '------------------------------------------------
}}

con
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  '_clkmode = xtal1 + pll8x
  '_xinfreq = 10_000_000

obj
#ifdef FDX
  sx : "FullDuplexSingleton"
#else
  sx : "SingleSerial"
#endif
  pj : "pjvmInterpreterCOG"
  pjd: "pjdata"
  pcn: "pjcon"
  pn : "pnative"
#ifdef GCOLLECT
  pm : "pjmemory"               ' best fit list power of 2 size
#endif

pub main | progptr, ctrlp, cmdptr
{{
Startup the JVM interpreter
}}
#ifdef FDX
  sx.start(31,30,0,19200)
#else
  sx.init(31,30,19200)
#endif
  waitcnt(clkfreq/1+cnt)
  'sx.str(string($d,"Hello JVM",$d))

  ctrlp := pjd.getControlPtr
  cmdptr := pjd.getCmdPtr

  progptr := pj.getProgPtr
  longmove(pj.getProgBuffer, progptr, pj.getProgSize/4)
  progptr := pj.getProgBuffer
  
  pjd.init(progptr,pj.getJavaCodeSize,1,false)
#ifdef GCOLLECT
  pm.init(pjd.getHS-2)
#endif

  cognew(handler, @hstack)

  pn.start(3)                ' start virtual peripherals
  pj.start(cmdptr)           ' start jvm engine

var long hstack[$40]

pub handler | cmd, tmp, cmdptr, ctlptr
{{
handle memory allocations, natives, and exceptions
}}
  cmdptr := pjd.getCmdPtr
  ctlptr := pjd.getControlPtr

  repeat

    if (long[ctlptr] > 0) and (long[ctlptr] & pj#MALLOC_BIT)
      tmp := long[ctlptr] & $3fff
#ifdef GCOLLECT
      pm.alloc(tmp)
#endif
      'pm.simplealloc(tmp)
      long[ctlptr] := -1

    if long[cmdptr] & pj#EXCPTN_BIT

      'cmd := getCmd(pjd.getProgPtr+pjd.getPC)
      sx.str(string($d,"Exception "))
      sx.hex(long[cmdptr],4)
      sx.str(string(" at "))
      sx.hex(pjd.getPC,4)
      sx.tx($d)
      long[cmdptr] := 0
      repeat ' stall forever

    elseif long[cmdptr] & $100

      cmd := long[cmdptr] & $3f
      pn.doNative(cmd)
      pjd.incrPC(3)
      long[cmdptr] := 0


